@@ -1,5 +1,5 @@
 /*
-   Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -9,14 +9,16 @@
    <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 */
 
+#include <sstream>
+#include <math.h>
+#include <stdio.h>
+
 #include "mysql_util.h"
 #include "mysql_art_resultset.h"
 #include "mysql_art_rset_metadata.h"
 #include "mysql_debug.h"
 
 #include <cppconn/exception.h>
-#include <sstream>
-#include <math.h>
 
 
 namespace sql
@@ -24,13 +26,13 @@ namespace sql
 namespace mysql
 {
 
-
 /* {{{ MyVal::MyVal() -I- */
-MyVal::MyVal(const std::string & s)
+MyVal::MyVal(const sql::SQLString & s)
   : val_type(typeString)
 {
-	val.str = new std::string(s);
+	val.str = new sql::SQLString(s);
 }
+
 /* }}} */
 
 
@@ -39,13 +41,13 @@ MyVal::MyVal(const char * const s)
   : val_type(typeString)
 {
 	// Init it clearly
-	val.str = new std::string(s);
+	val.str = new sql::SQLString(s);
 }
 /* }}} */
 
 
 /* {{{ MyVal::getString() -I- */
-std::string
+sql::SQLString
 MyVal::getString()
 {
 	switch (val_type) {
@@ -55,25 +57,25 @@ MyVal::getString()
 		{
 			char buf[31];
 			size_t len = snprintf(buf, sizeof(buf) - 1, "%14.14Lf", val.dval);
-			return std::string(buf, len);
+			return sql::SQLString(buf, len);
 		}
 		case typeInt:
 		{
 			char buf[20];
 			size_t len = snprintf(buf, sizeof(buf) - 1, "%lld", (long long int)val.lval);
-			return std::string(buf, len);
+			return sql::SQLString(buf, len);
 		}
 		case typeUInt:
 		{
 			char buf[20];
 			size_t len = snprintf(buf, sizeof(buf) - 1, "%llu", (long long unsigned int)val.ulval);
-			return std::string(buf, len);
+			return sql::SQLString(buf, len);
 		}
 		case typeBool:
 		{
 			char buf[3];
 			size_t len = snprintf(buf, sizeof(buf) - 1, "%d", val.bval);
-			return std::string(buf, len);
+			return sql::SQLString(buf, len);
 		}
 		case typePtr:
 			return "";
@@ -179,22 +181,21 @@ MyVal::getBool()
 
 
 /* {{{ MySQL_ArtResultSet::MySQL_ArtResultSet() -I- */
-MySQL_ArtResultSet::MySQL_ArtResultSet(const StringList& fn, rset_t * const rs, sql::mysql::util::my_shared_ptr< MySQL_DebugLogger > * l)
-  : rset(rs), current_record(rset->begin()), started(false), row_position(0), is_closed(false), logger(l? l->getReference():NULL)
+MySQL_ArtResultSet::MySQL_ArtResultSet(const StringList& fn, rset_t * const rs, boost::shared_ptr< MySQL_DebugLogger > & l)
+  : num_fields(static_cast<int>(fn.size())), rset(rs), current_record(rset->begin()),
+    started(false), field_index_to_name_map(new sql::SQLString[num_fields]),
+	num_rows(rset->size()), row_position(0), is_closed(false), logger(l)
 {
 	CPP_ENTER("MySQL_ArtResultSet::MySQL_ArtResultSet");
 	CPP_INFO_FMT("metadata.size=%d resultset.size=%d", fn.size(), rset->size());
-	num_fields = static_cast<int>(fn.size());
 
-	num_rows = rset->size();
-
-	field_index_to_name_map = new std::string[num_fields];
+//	field_index_to_name_map = new sql::SQLString[num_fields];
 
 	unsigned int idx = 0;
 	for (StringList::const_iterator it = fn.begin(), e = fn.end(); it != e; ++it, ++idx) {
-		sql::mysql::util::my_array_guard< char > upstring(sql::mysql::util::utf8_strup(it->c_str(), 0));
-		field_name_to_index_map[std::string(upstring.get())] = idx;
-		field_index_to_name_map[idx] = std::string(upstring.get());
+		boost::scoped_array< char > upstring(sql::mysql::util::utf8_strup(it->c_str(), 0));
+		field_name_to_index_map[sql::SQLString(upstring.get())] = idx;
+		field_index_to_name_map[idx] = upstring.get();
 	}
 
 	meta.reset(new MySQL_ArtResultSetMetaData(this, logger));
@@ -205,14 +206,10 @@ MySQL_ArtResultSet::MySQL_ArtResultSet(const StringList& fn, rset_t * const rs,
 /* {{{ MySQL_ArtResultSet::~MySQL_ArtResultSet() -I- */
 MySQL_ArtResultSet::~MySQL_ArtResultSet()
 {
-	/* Don't remove the block or we can get into problems with logger */
-	{
-		CPP_ENTER("MySQL_ArtResultSet::~MySQL_ArtResultSet");
-		if (!isClosed()) {
-			close();
-		}
+	CPP_ENTER("MySQL_ArtResultSet::~MySQL_ArtResultSet");
+	if (!isClosed()) {
+		close();
 	}
-	logger->freeReference();
 }
 /* }}} */
 
@@ -296,6 +293,17 @@ MySQL_ArtResultSet::beforeFirst()
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::cancelRowUpdates() -U- */
+void
+MySQL_ArtResultSet::cancelRowUpdates()
+{
+	CPP_ENTER("MySQL_ArtResultSet::cancelRowUpdates");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::cancelRowUpdates()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::checkValid() -I- */
 void
 MySQL_ArtResultSet::checkValid() const
@@ -309,13 +317,24 @@ MySQL_ArtResultSet::checkValid() const
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::clearWarnings() -U- */
+void
+MySQL_ArtResultSet::clearWarnings()
+{
+	CPP_ENTER("MySQL_ArtResultSet::clearWarnings");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::clearWarnings()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::close() -I- */
 void
 MySQL_ArtResultSet::close()
 {
 	CPP_ENTER("MySQL_ArtResultSet::close");
 	checkValid();
-	delete [] field_index_to_name_map;
+//	delete [] field_index_to_name_map;
 	is_closed = true;
 }
 /* }}} */
@@ -323,12 +342,13 @@ MySQL_ArtResultSet::close()
 
 /* {{{ MySQL_ArtResultSet::findColumn() -I- */
 uint32_t
-MySQL_ArtResultSet::findColumn(const std::string& columnLabel) const
+MySQL_ArtResultSet::findColumn(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::columnLabel");
 	checkValid();
 
-	sql::mysql::util::my_array_guard< char > upstring(sql::mysql::util::utf8_strup(columnLabel.c_str(), 0));
+	boost::scoped_array< char > upstring(sql::mysql::util::utf8_strup(columnLabel.c_str(), 0));
+
 	FieldNameIndexMap::const_iterator iter = field_name_to_index_map.find(upstring.get());
 
 	if (iter == field_name_to_index_map.end()) {
@@ -372,7 +392,7 @@ MySQL_ArtResultSet::getBlob(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::getBlob() -I- */
 std::istream *
-MySQL_ArtResultSet::getBlob(const std::string& columnLabel) const
+MySQL_ArtResultSet::getBlob(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getBlob(string)");
 	return new std::istringstream(getString(columnLabel));
@@ -397,7 +417,7 @@ MySQL_ArtResultSet::getBoolean(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::getBoolean() -I- */
 bool
-MySQL_ArtResultSet::getBoolean(const std::string& columnLabel) const
+MySQL_ArtResultSet::getBoolean(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getBoolean(string)");
 	return getInt(columnLabel) != 0;
@@ -405,6 +425,30 @@ MySQL_ArtResultSet::getBoolean(const std::string& columnLabel) const
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::getConcurrency() -U- */
+int
+MySQL_ArtResultSet::getConcurrency()
+{
+	CPP_ENTER("MySQL_ArtResultSet::getConcurrency");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getConcurrency()");
+	return 0; // This will shut up compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::getCursorName() -U- */
+SQLString
+MySQL_ArtResultSet::getCursorName()
+{
+	CPP_ENTER("MySQL_ArtResultSet::getCursorName");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getCursorName()");
+	return 0; // This will shut up compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::getDouble() -I- */
 long double
 MySQL_ArtResultSet::getDouble(uint32_t columnIndex) const
@@ -427,7 +471,7 @@ MySQL_ArtResultSet::getDouble(uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::getDouble() -I- */
 long double
-MySQL_ArtResultSet::getDouble(const std::string& columnLabel) const
+MySQL_ArtResultSet::getDouble(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getDouble(string)");
 	return getDouble(findColumn(columnLabel));
@@ -435,6 +479,42 @@ MySQL_ArtResultSet::getDouble(const std::string& columnLabel) const
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::getFetchDirection() -U- */
+int
+MySQL_ArtResultSet::getFetchDirection()
+{
+	CPP_ENTER("MySQL_ArtResultSet::getFetchDirection");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getFetchDirection()");
+	return 0; // This will shut up compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::getFetchSize() -U- */
+size_t
+MySQL_ArtResultSet::getFetchSize()
+{
+	CPP_ENTER("MySQL_ArtResultSet::getFetchSize");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getFetchSize()");
+	return 0; // This will shut up compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::getHoldability() -U- */
+int
+MySQL_ArtResultSet::getHoldability()
+{
+	CPP_ENTER("MySQL_ArtResultSet::getHoldability");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getHoldability()");
+	return 0; // This will shut up compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::getInt() -I- */
 int32_t
 MySQL_ArtResultSet::getInt(uint32_t columnIndex) const
@@ -447,7 +527,7 @@ MySQL_ArtResultSet::getInt(uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::getInt() -I- */
 int32_t
-MySQL_ArtResultSet::getInt(const std::string& columnLabel) const
+MySQL_ArtResultSet::getInt(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getInt(string)");
 	return getInt(findColumn(columnLabel));
@@ -467,7 +547,7 @@ MySQL_ArtResultSet::getUInt(uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::getUInt() -I- */
 uint32_t
-MySQL_ArtResultSet::getUInt(const std::string& columnLabel) const
+MySQL_ArtResultSet::getUInt(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getUInt(string)");
 	return getUInt(findColumn(columnLabel));
@@ -497,7 +577,7 @@ MySQL_ArtResultSet::getInt64(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::getInt64() -I- */
 int64_t
-MySQL_ArtResultSet::getInt64(const std::string& columnLabel) const
+MySQL_ArtResultSet::getInt64(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getInt64(string)");
 	return getInt64(findColumn(columnLabel));
@@ -527,7 +607,7 @@ MySQL_ArtResultSet::getUInt64(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::getUInt64() -I- */
 uint64_t
-MySQL_ArtResultSet::getUInt64(const std::string& columnLabel) const
+MySQL_ArtResultSet::getUInt64(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getUInt64(string)");
 	return getUInt64(findColumn(columnLabel));
@@ -558,6 +638,30 @@ MySQL_ArtResultSet::getRow() const
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::getRowId() -U- */
+sql::RowID *
+MySQL_ArtResultSet::getRowId(uint32_t)
+{
+	CPP_ENTER("MySQL_ArtResultSet::getRowId");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getRowId()");
+	return NULL; // This will shut up compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::getRowId() -U- */
+sql::RowID *
+MySQL_ArtResultSet::getRowId(const sql::SQLString &)
+{
+	CPP_ENTER("MySQL_ArtResultSet::getRowId");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getRowId()");
+	return NULL; // This will shut up compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::getStatement() -I- */
 const Statement *
 MySQL_ArtResultSet::getStatement() const
@@ -570,7 +674,7 @@ MySQL_ArtResultSet::getStatement() const
 
 
 /* {{{ MySQL_ArtResultSet::getString() -I- */
-std::string
+SQLString
 MySQL_ArtResultSet::getString(uint32_t columnIndex) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getString(int)");
@@ -591,8 +695,8 @@ MySQL_ArtResultSet::getString(uint32_t columnIndex) const
 
 
 /* {{{ MySQL_ArtResultSet::getString() -I- */
-std::string
-MySQL_ArtResultSet::getString(const std::string& columnLabel) const
+SQLString
+MySQL_ArtResultSet::getString(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::getString(string)");
 	return getString(findColumn(columnLabel));
@@ -611,6 +715,29 @@ MySQL_ArtResultSet::getType() const
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::getWarnings() -U- */
+void
+MySQL_ArtResultSet::getWarnings()
+{
+	CPP_ENTER("MySQL_ArtResultSet::getWarnings");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getWarnings()");
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::insertRow() -U- */
+void
+MySQL_ArtResultSet::insertRow()
+{
+	CPP_ENTER("MySQL_ArtResultSet::insertRow");
+	checkValid();
+	 /* TODO - We don't support inserting anyway */
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::insertRow()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::isAfterLast() -I- */
 bool
 MySQL_ArtResultSet::isAfterLast() const
@@ -687,7 +814,7 @@ MySQL_ArtResultSet::isNull(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ArtResultSet::isNull() -I- */
 bool
-MySQL_ArtResultSet::isNull(const std::string& columnLabel) const
+MySQL_ArtResultSet::isNull(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ArtResultSet::isNull(string)");
 	return isNull(findColumn(columnLabel));
@@ -705,7 +832,29 @@ MySQL_ArtResultSet::last()
 		row_position = num_rows;
 		seek();
 	}
-	return num_rows? true:false;
+	return num_rows != 0;
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::moveToCurrentRow() -U- */
+void
+MySQL_ArtResultSet::moveToCurrentRow()
+{
+	CPP_ENTER("MySQL_ArtResultSet::moveToCurrentRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::moveToCurrentRow()");
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::moveToInsertRow() -U- */
+void
+MySQL_ArtResultSet::moveToInsertRow()
+{
+	CPP_ENTER("MySQL_ArtResultSet::moveToInsertRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::moveToInsertRow()");
 }
 /* }}} */
 
@@ -755,6 +904,17 @@ MySQL_ArtResultSet::previous()
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::refreshRow() -U- */
+void
+MySQL_ArtResultSet::refreshRow()
+{
+	CPP_ENTER("MySQL_ArtResultSet::refreshRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::refreshRow()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::relative() -I- */
 bool
 MySQL_ArtResultSet::relative(const int rows)
@@ -775,6 +935,42 @@ MySQL_ArtResultSet::relative(const int rows)
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::rowDeleted() -U- */
+bool
+MySQL_ArtResultSet::rowDeleted()
+{
+	CPP_ENTER("MySQL_ArtResultSet::rowDeleted");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::rowDeleted()");
+	return false; // This will shut up compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::rowInserted() -U- */
+bool
+MySQL_ArtResultSet::rowInserted()
+{
+	CPP_ENTER("MySQL_ArtResultSet::rowInserted");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::rowInserted()");
+	return false; // This will shut up compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::rowUpdated() -U- */
+bool
+MySQL_ArtResultSet::rowUpdated()
+{
+	CPP_ENTER("MySQL_ArtResultSet::rowUpdated");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ArtResultSet::rowUpdated()");
+	return false; // This will shut up compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::rowsCount() -I- */
 size_t
 MySQL_ArtResultSet::rowsCount() const
@@ -786,6 +982,17 @@ MySQL_ArtResultSet::rowsCount() const
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::setFetchSize() -U- */
+void
+MySQL_ArtResultSet::setFetchSize(size_t /* rows */)
+{
+	CPP_ENTER("MySQL_ArtResultSet::setFetchSize");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::rowDeleted()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::wasNull() -I- */
 bool
 MySQL_ArtResultSet::wasNull() const
