@@ -12,27 +12,46 @@
 #ifndef _MYSQL_PARAMETER_METADATA_H_
 #define _MYSQL_PARAMETER_METADATA_H_
 
-#include <string>
+#include <boost/shared_ptr.hpp>
 
 #include <cppconn/parameter_metadata.h>
 
-#include "mysql_private_iface.h"
-
 namespace sql
 {
 namespace mysql
 {
+namespace NativeAPI
+{
+	class NativeStatementWrapper;
+}
 
 class MySQL_ParameterMetaData : public sql::ParameterMetaData
 {
 	unsigned int param_count;
 public:
-	MySQL_ParameterMetaData(const MYSQL_STMT * const stmt);
+	MySQL_ParameterMetaData( boost::shared_ptr< NativeAPI::NativeStatementWrapper > & stmt);
 
 	virtual ~MySQL_ParameterMetaData() {}
 
+	virtual sql::SQLString getParameterClassName(unsigned int paramNo);
+
 	virtual int getParameterCount();
 
+	virtual int getParameterMode(unsigned int paramNo);
+
+	virtual int getParameterType(unsigned int paramNo);
+
+	virtual sql::SQLString getParameterTypeName(unsigned int paramNo);
+
+	virtual int getPrecision(unsigned int paramNo);
+
+	virtual int getScale(unsigned int paramNo);
+
+	virtual int isNullable(unsigned int paramNo);
+
+	virtual bool isSigned(unsigned int paramNo);
+
+
 private:
 	/* Prevent use of these */
 	MySQL_ParameterMetaData(const MySQL_ParameterMetaData &);
