@@ -1,5 +1,5 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2007 - 2008 MySQL AB, 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -13,6 +13,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <sstream>
+#include <boost/scoped_array.hpp>
 
 
 #include <cppconn/exception.h>
@@ -25,6 +26,8 @@
 #include "mysql_ps_resultset_metadata.h"
 #include "mysql_resultbind.h"
 
+#include "nativeapi/native_statement_wrapper.h"
+#include "nativeapi/native_resultset_wrapper.h"
 
 #include "mysql_debug.h"
 
@@ -63,31 +66,30 @@ static inline char * my_f_to_a(char * buf, size_t buf_size, double a)
 
 /* {{{ MySQL_Prepared_ResultSet::MySQL_Prepared_ResultSet() -I- */
 MySQL_Prepared_ResultSet::MySQL_Prepared_ResultSet(
-			MYSQL_STMT * s,
-			MySQL_ResultBind * r_bind,
+			boost::shared_ptr< NativeAPI::NativeStatementWrapper > & s,
+			boost::shared_ptr< MySQL_ResultBind > & r_bind,
 			sql::ResultSet::enum_type rset_type,
 			MySQL_Prepared_Statement * par,
-			sql::mysql::util::my_shared_ptr< MySQL_DebugLogger > * l
+			boost::shared_ptr< MySQL_DebugLogger > & l
 		)
-	: row(NULL), stmt(s), row_position(0), parent(par), is_valid(true),
-	  logger(l? l->getReference():NULL), result_bind(r_bind),
-	  resultset_type(rset_type)
+	: row(NULL), proxy(s), row_position(0), parent(par), is_valid(true),
+		logger(l), result_bind(r_bind), resultset_type(rset_type)
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::MySQL_Prepared_ResultSet");
+
 	result_bind->bindResult();
 
-	MYSQL_RES * result_meta = mysql_stmt_result_metadata(stmt);
-	num_fields = mysql_stmt_field_count(stmt);
-	num_rows = mysql_stmt_num_rows(stmt);
+	boost::scoped_ptr< NativeAPI::NativeResultsetWrapper > result_meta( proxy->result_metadata() );
+	num_fields = proxy->field_count();
+	num_rows = proxy->num_rows();
 
 	CPP_INFO_FMT("num_fields=%u num_rows=%u", num_fields, num_rows);
 	for (unsigned int i = 0; i < num_fields; ++i) {
-		sql::mysql::util::my_array_guard< char > upstring(sql::mysql::util::utf8_strup(mysql_fetch_field(result_meta)->name, 0));
-		field_name_to_index_map[std::string(upstring.get())] = i;
+		boost::scoped_array< char > upstring(sql::mysql::util::utf8_strup(result_meta->fetch_field()->name, 0));
+		field_name_to_index_map[sql::SQLString(upstring.get())] = i;
 	}
-	mysql_free_result(result_meta);
-	result_meta = NULL;
-	rs_meta.reset(new MySQL_Prepared_ResultSetMetaData(stmt, logger));
+
+	rs_meta.reset(new MySQL_PreparedResultSetMetaData(proxy, logger));
 }
 /* }}} */
 
@@ -95,12 +97,8 @@ MySQL_Prepared_ResultSet::MySQL_Prepared_ResultSet(
 /* {{{ MySQL_Prepared_ResultSet::~MySQL_Prepared_ResultSet() -I- */
 MySQL_Prepared_ResultSet::~MySQL_Prepared_ResultSet()
 {
-	/* Don't remove the block or we can get into problems with logger */
-	{
-		CPP_ENTER("MySQL_Prepared_ResultSet::~MySQL_Prepared_ResultSet");
-		closeIntern();
-	}
-	logger->freeReference();
+	CPP_ENTER("MySQL_Prepared_ResultSet::~MySQL_Prepared_ResultSet");
+	closeIntern();
 }
 /* }}} */
 
@@ -158,12 +156,23 @@ MySQL_Prepared_ResultSet::beforeFirst()
 	CPP_ENTER("MySQL_Prepared_ResultSet::beforeFirst");
 	checkValid();
 	checkScrollable();
-	mysql_stmt_data_seek(stmt, 0);
+	proxy->data_seek(0);
 	row_position = 0;
 }
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::cancelRowUpdates() -U- */
+void
+MySQL_Prepared_ResultSet::cancelRowUpdates()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::cancelRowUpdates");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::cancelRowUpdates()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::checkValid() -I- */
 void
 MySQL_Prepared_ResultSet::checkValid() const
@@ -201,6 +210,17 @@ MySQL_Prepared_ResultSet::isScrollable() const
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::clearWarnings() -U- */
+void
+MySQL_Prepared_ResultSet::clearWarnings()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::clearWarnings");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::clearWarnings()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::close() -I- */
 void
 MySQL_Prepared_ResultSet::close()
@@ -224,11 +244,11 @@ MySQL_Prepared_ResultSet::closeIntern()
 
 /* {{{ MySQL_Prepared_ResultSet::findColumn() -I- */
 uint32_t
-MySQL_Prepared_ResultSet::findColumn(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::findColumn(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::findColumn");
 	checkValid();
-	sql::mysql::util::my_array_guard< char > upstring(sql::mysql::util::utf8_strup(columnLabel.c_str(), 0));
+	boost::scoped_array< char > upstring(sql::mysql::util::utf8_strup(columnLabel.c_str(), 0));
 	FieldNameIndexMap::const_iterator iter= field_name_to_index_map.find(upstring.get());
 
 	if (iter == field_name_to_index_map.end()) {
@@ -273,7 +293,7 @@ MySQL_Prepared_ResultSet::getBlob(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::getBlob() -I- */
 std::istream *
-MySQL_Prepared_ResultSet::getBlob(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::getBlob(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getBlob(string)");
 	/* isBeforeFirst checks for validity */
@@ -302,7 +322,7 @@ MySQL_Prepared_ResultSet::getBoolean(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::getBoolean() -I- */
 bool
-MySQL_Prepared_ResultSet::getBoolean(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::getBoolean(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getBoolean(string)");
 	/* isBeforeFirst checks for validity */
@@ -314,6 +334,30 @@ MySQL_Prepared_ResultSet::getBoolean(const std::string& columnLabel) const
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::getConcurrency() -U- */
+int
+MySQL_Prepared_ResultSet::getConcurrency()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getConcurrency");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getConcurrency()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::getCursorName() -U- */
+SQLString
+MySQL_Prepared_ResultSet::getCursorName()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getCursorName");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getCursorName()");
+	return ""; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::getDouble() -I- */
 long double
 MySQL_Prepared_ResultSet::getDouble(const uint32_t columnIndex) const
@@ -403,7 +447,7 @@ MySQL_Prepared_ResultSet::getDouble(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::getDouble() -I- */
 long double
-MySQL_Prepared_ResultSet::getDouble(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::getDouble(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getDouble(string)");
 	return getDouble(findColumn(columnLabel));
@@ -411,6 +455,42 @@ MySQL_Prepared_ResultSet::getDouble(const std::string& columnLabel) const
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::getFetchDirection() -U- */
+int
+MySQL_Prepared_ResultSet::getFetchDirection()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getFetchDirection");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getFetchDirection()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::getFetchSize() -U- */
+size_t
+MySQL_Prepared_ResultSet::getFetchSize()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getFetchSize");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getFetchSize()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::getHoldability() -U- */
+int
+MySQL_Prepared_ResultSet::getHoldability()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getHoldability");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getHoldability()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::getInt() -I- */
 int32_t
 MySQL_Prepared_ResultSet::getInt(const uint32_t columnIndex) const
@@ -430,7 +510,7 @@ MySQL_Prepared_ResultSet::getInt(const uint32_t columnIndex) const
 	if (*result_bind->rbind[columnIndex - 1].is_null) {
 		return 0;
 	}
-	
+
 	return static_cast<int32_t>(getInt64_intern(columnIndex, true));
 }
 /* }}} */
@@ -438,7 +518,7 @@ MySQL_Prepared_ResultSet::getInt(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::getInt() -I- */
 int32_t
-MySQL_Prepared_ResultSet::getInt(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::getInt(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getInt(string)");
 	return getInt(findColumn(columnLabel));
@@ -472,7 +552,7 @@ MySQL_Prepared_ResultSet::getUInt(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::getUInt() -I- */
 uint32_t
-MySQL_Prepared_ResultSet::getUInt(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::getUInt(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getUInt(string)");
 	return getUInt(findColumn(columnLabel));
@@ -508,6 +588,21 @@ MySQL_Prepared_ResultSet::getInt64_intern(const uint32_t columnIndex, bool /* cu
 			CPP_INFO("It's a string");
 			return strtoll(getString(columnIndex).c_str(), NULL, 10);
 		case sql::DataType::BIT:
+		{
+			int64_t uval = 0;
+			/* This length is in bytes, on the contrary to what can be seen in mysql_resultset.cpp where the Meta is used */
+			switch (*result_bind->rbind[columnIndex - 1].length) {
+				case 8:uval = (int64_t) bit_uint8korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 7:uval = (int64_t) bit_uint7korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 6:uval = (int64_t) bit_uint6korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 5:uval = (int64_t) bit_uint5korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 4:uval = (int64_t) bit_uint4korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 3:uval = (int64_t) bit_uint3korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 2:uval = (int64_t) bit_uint2korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 1:uval = (int64_t) bit_uint1korr(result_bind->rbind[columnIndex - 1].buffer);break;
+			}
+			return uval;
+		}
 		case sql::DataType::YEAR:	// fetched as a SMALLINT
 		case sql::DataType::TINYINT:
 		case sql::DataType::SMALLINT:
@@ -604,7 +699,7 @@ MySQL_Prepared_ResultSet::getInt64(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::getInt64() -I- */
 int64_t
-MySQL_Prepared_ResultSet::getInt64(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::getInt64(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getInt64(string)");
 	return getInt64(findColumn(columnLabel));
@@ -640,6 +735,21 @@ MySQL_Prepared_ResultSet::getUInt64_intern(const uint32_t columnIndex, bool /* c
 			CPP_INFO("It's a string");
 			return strtoull(getString(columnIndex).c_str(), NULL, 10);
 		case sql::DataType::BIT:
+		{
+			uint64_t uval = 0;
+			/* This length is in bytes, on the contrary to what can be seen in mysql_resultset.cpp where the Meta is used */
+			switch (*result_bind->rbind[columnIndex - 1].length) {
+				case 8:uval = (uint64_t) bit_uint8korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 7:uval = (uint64_t) bit_uint7korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 6:uval = (uint64_t) bit_uint6korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 5:uval = (uint64_t) bit_uint5korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 4:uval = (uint64_t) bit_uint4korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 3:uval = (uint64_t) bit_uint3korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 2:uval = (uint64_t) bit_uint2korr(result_bind->rbind[columnIndex - 1].buffer);break;
+				case 1:uval = (uint64_t) bit_uint1korr(result_bind->rbind[columnIndex - 1].buffer);break;
+			}
+			return uval;
+		}
 		case sql::DataType::YEAR:	// fetched as a SMALLINT
 		case sql::DataType::TINYINT:
 		case sql::DataType::SMALLINT:
@@ -682,13 +792,13 @@ MySQL_Prepared_ResultSet::getUInt64_intern(const uint32_t columnIndex, bool /* c
 					if (is_it_unsigned) {
 						ret =  !is_it_null? *reinterpret_cast<uint64_t *>(result_bind->rbind[columnIndex - 1].buffer):0;
 					} else {
-						ret =  !is_it_null? *reinterpret_cast<int64_t *>(result_bind->rbind[columnIndex - 1].buffer):0;				
+						ret =  !is_it_null? *reinterpret_cast<int64_t *>(result_bind->rbind[columnIndex - 1].buffer):0;
 #if WE_WANT_TO_SEE_MORE_FAILURES_IN_UNIT_RESULTSET
 						if (is_it_null) {
 							if (cutTooBig && *reinterpret_cast<int64_t *>(result_bind->rbind[columnIndex - 1].buffer) < 0) {
-								ret =  *reinterpret_cast<uint64_t *>(result_bind->rbind[columnIndex - 1].buffer);				
+								ret =  *reinterpret_cast<uint64_t *>(result_bind->rbind[columnIndex - 1].buffer);
 							} else {
-								ret =  *reinterpret_cast<int64_t *>(result_bind->rbind[columnIndex - 1].buffer);				
+								ret =  *reinterpret_cast<int64_t *>(result_bind->rbind[columnIndex - 1].buffer);
 							}
 						} else {
 							ret = 0;
@@ -740,7 +850,7 @@ MySQL_Prepared_ResultSet::getUInt64(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::getUInt64() -I- */
 uint64_t
-MySQL_Prepared_ResultSet::getUInt64(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::getUInt64(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getUInt64(string)");
 	return getUInt64(findColumn(columnLabel));
@@ -771,6 +881,30 @@ MySQL_Prepared_ResultSet::getRow() const
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::getRowId() -U- */
+sql::RowID *
+MySQL_Prepared_ResultSet::getRowId(uint32_t)
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getRowId");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getRowId()");
+	return NULL; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::getRowId() -U- */
+sql::RowID *
+MySQL_Prepared_ResultSet::getRowId(const sql::SQLString &)
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getRowId");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getRowId()");
+	return NULL; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::getStatement() -I- */
 const sql::Statement *
 MySQL_Prepared_ResultSet::getStatement() const
@@ -782,7 +916,7 @@ MySQL_Prepared_ResultSet::getStatement() const
 
 
 /* {{{ MySQL_Prepared_ResultSet::getString() -I- */
-std::string
+SQLString
 MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getString(int)");
@@ -799,7 +933,7 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 
 	last_queried_column = columnIndex;
 	if (*result_bind->rbind[columnIndex - 1].is_null) {
-		return std::string("");
+		return sql::SQLString("");
 	}
 
 	switch (rs_meta->getColumnType(columnIndex)) {
@@ -809,7 +943,7 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 			MYSQL_TIME * t = static_cast<MYSQL_TIME *>(result_bind->rbind[columnIndex - 1].buffer);
 			snprintf(buf, sizeof(buf) - 1, "%04d-%02d-%02d %02d:%02d:%02d", t->year, t->month, t->day, t->hour, t->minute, t->second);
 			CPP_INFO_FMT("It's a datetime/timestamp %s", buf);
-			return std::string(buf);
+			return sql::SQLString(buf);
 		}
 		case sql::DataType::DATE:
 		{
@@ -817,7 +951,7 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 			MYSQL_TIME * t = static_cast<MYSQL_TIME *>(result_bind->rbind[columnIndex - 1].buffer);
 			snprintf(buf, sizeof(buf) - 1, "%02d-%02d-%02d", t->year, t->month, t->day);
 			CPP_INFO_FMT("It's a date %s", buf);
-			return std::string(buf);
+			return sql::SQLString(buf);
 		}
 		case sql::DataType::TIME:
 		{
@@ -825,7 +959,7 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 			MYSQL_TIME * t = static_cast<MYSQL_TIME *>(result_bind->rbind[columnIndex - 1].buffer);
 			snprintf(buf, sizeof(buf) - 1, "%s%02d:%02d:%02d", t->neg? "-":"", t->hour, t->minute, t->second);
 			CPP_INFO_FMT("It's a time %s", buf);
-			return std::string(buf);
+			return sql::SQLString(buf);
 		}
 		case sql::DataType::BIT:
 		case sql::DataType::YEAR:	// fetched as a SMALLINT
@@ -840,9 +974,9 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 			if (result_bind->rbind[columnIndex - 1].is_unsigned) {
 				my_ul_to_a(buf, sizeof(buf) - 1, getUInt64_intern(columnIndex, false));
 			} else {
-				my_l_to_a(buf, sizeof(buf) - 1, getInt64_intern(columnIndex, false));			
+				my_l_to_a(buf, sizeof(buf) - 1, getInt64_intern(columnIndex, false));
 			}
-			return std::string(buf);
+			return sql::SQLString(buf);
 		}
 		case sql::DataType::REAL:
 		case sql::DataType::DOUBLE:
@@ -850,7 +984,7 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 			char buf[50];
 			CPP_INFO("It's a double");
 			my_f_to_a(buf, sizeof(buf) - 1, getDouble(columnIndex));
-			return std::string(buf);
+			return sql::SQLString(buf);
 		}
 		case sql::DataType::NUMERIC:
 		case sql::DataType::DECIMAL:
@@ -863,9 +997,9 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 		case sql::DataType::SET:
 		case sql::DataType::ENUM:
 			CPP_INFO("It's a string");
-			return  std::string(static_cast<char *>(result_bind->rbind[columnIndex - 1].buffer), *result_bind->rbind[columnIndex - 1].length);
+			return  sql::SQLString(static_cast<char *>(result_bind->rbind[columnIndex - 1].buffer), *result_bind->rbind[columnIndex - 1].length);
 		default:
-			break;		
+			break;
 		// ToDo : Geometry? default ?
 	}
 
@@ -877,8 +1011,8 @@ MySQL_Prepared_ResultSet::getString(const uint32_t columnIndex) const
 
 
 /* {{{ MySQL_Prepared_ResultSet::getString() -I- */
-std::string
-MySQL_Prepared_ResultSet::getString(const std::string& columnLabel) const
+SQLString
+MySQL_Prepared_ResultSet::getString(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::getString(string)");
 	return getString(findColumn(columnLabel));
@@ -897,6 +1031,17 @@ MySQL_Prepared_ResultSet::getType() const
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::getWarnings() -U- */
+void
+MySQL_Prepared_ResultSet::getWarnings()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::getWarnings");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::getWarnings()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::isAfterLast() -I- */
 bool
 MySQL_Prepared_ResultSet::isAfterLast() const
@@ -929,6 +1074,17 @@ MySQL_Prepared_ResultSet::isClosed() const
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::insertRow() -U- */
+void
+MySQL_Prepared_ResultSet::insertRow()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::beforeFirst");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::insertRow()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::isFirst() -I- */
 bool
 MySQL_Prepared_ResultSet::isFirst() const
@@ -972,7 +1128,7 @@ MySQL_Prepared_ResultSet::isNull(const uint32_t columnIndex) const
 
 /* {{{ MySQL_Prepared_ResultSet::isNull() -I- */
 bool
-MySQL_Prepared_ResultSet::isNull(const std::string& columnLabel) const
+MySQL_Prepared_ResultSet::isNull(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::isNull(string)");
 	uint32_t col_idx = findColumn(columnLabel);
@@ -1000,6 +1156,30 @@ MySQL_Prepared_ResultSet::last()
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::moveToCurrentRow() -U- */
+void
+MySQL_Prepared_ResultSet::moveToCurrentRow()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::moveToCurrentRow");
+	checkValid();
+	checkScrollable();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::moveToCurrentRow()");
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::moveToInsertRow() -U- */
+void
+MySQL_Prepared_ResultSet::moveToInsertRow()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::moveToInsertRow");
+	checkValid();
+	checkScrollable();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::moveToInsertRow()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::next() -I- */
 bool
 MySQL_Prepared_ResultSet::next()
@@ -1014,8 +1194,21 @@ MySQL_Prepared_ResultSet::next()
 			++row_position;
 			ret = false;
 		} else if (row_position < num_rows + 1) {
-			mysql_stmt_data_seek(stmt, row_position);
-			int result = mysql_stmt_fetch(stmt);
+
+			if (row_position == 0) {
+				proxy->data_seek(row_position);
+			} else {
+				/*
+				NOTE: Buffered only optimization.
+
+				If our position is anything else but beforeFirst/afterLast, then the last
+				cursor operation ended with a mysql_stmt_fetch call, which by definition (?)
+				moves the cursor to the next row after the fetched one. So, we do not need
+				any positioning here.
+				*/
+			}
+
+			int result = proxy->fetch();
 			if (!result || result == MYSQL_DATA_TRUNCATED) {
 				ret = true;
 			}
@@ -1026,7 +1219,7 @@ MySQL_Prepared_ResultSet::next()
 		}
 		CPP_INFO_FMT("new_row_position=%llu ret=%d", row_position, ret);
 	} else {
-		int result = mysql_stmt_fetch(stmt);
+		int result = proxy->fetch();
 		if (!result || result == MYSQL_DATA_TRUNCATED) {
 			ret = true;
 		}
@@ -1053,8 +1246,8 @@ MySQL_Prepared_ResultSet::previous()
 		return false;
 	} else if (row_position > 1) {
 		--row_position;
-		mysql_stmt_data_seek(stmt, row_position - 1);
-		int result = mysql_stmt_fetch(stmt);
+		proxy->data_seek(row_position - 1);
+		int result = proxy->fetch();
 		if (!result || result == MYSQL_DATA_TRUNCATED) {
 			return true;
 		}
@@ -1068,6 +1261,17 @@ MySQL_Prepared_ResultSet::previous()
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::refreshRow() -U- */
+void
+MySQL_Prepared_ResultSet::refreshRow()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::refreshRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::refreshRow()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::relative() -I- */
 bool
 MySQL_Prepared_ResultSet::relative(const int rows)
@@ -1080,7 +1284,7 @@ MySQL_Prepared_ResultSet::relative(const int rows)
 			row_position = rows > 0? num_rows + 1 : 0; /* after last or before first */
 		} else {
 			row_position += rows;
-			mysql_stmt_data_seek(stmt, row_position - 1);
+			proxy->data_seek(row_position - 1);
 		}
 	}
 
@@ -1089,6 +1293,42 @@ MySQL_Prepared_ResultSet::relative(const int rows)
 /* }}} */
 
 
+/* {{{ MySQL_Prepared_ResultSet::rowDeleted() -U- */
+bool
+MySQL_Prepared_ResultSet::rowDeleted()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::rowDeleted");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::rowDeleted()");
+	return false; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::rowInserted() -U- */
+bool
+MySQL_Prepared_ResultSet::rowInserted()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::rowInserted");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::rowInserted()");
+	return false; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::rowUpdated() -U- */
+bool
+MySQL_Prepared_ResultSet::rowUpdated()
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::rowUpdated");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::rowUpdated()");
+	return false; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_Prepared_ResultSet::rowsCount() -I- */
 size_t
 MySQL_Prepared_ResultSet::rowsCount() const
@@ -1096,7 +1336,18 @@ MySQL_Prepared_ResultSet::rowsCount() const
 	CPP_ENTER("MySQL_Prepared_ResultSet::rowsCount");
 	checkValid();
 	checkScrollable();
-	return (size_t) mysql_stmt_num_rows(stmt);
+	return static_cast<size_t>(proxy->num_rows());
+}
+/* }}} */
+
+
+/* {{{ MySQL_Prepared_ResultSet::setFetchSize() -U- */
+void
+MySQL_Prepared_ResultSet::setFetchSize(size_t /* rows */)
+{
+	CPP_ENTER("MySQL_Prepared_ResultSet::setFetchSize");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_Prepared_ResultSet::setFetchSize()");
 }
 /* }}} */
 
@@ -1132,8 +1383,8 @@ void
 MySQL_Prepared_ResultSet::seek()
 {
 	CPP_ENTER("MySQL_Prepared_ResultSet::seek");
-	mysql_stmt_data_seek(stmt, row_position - 1);
-	mysql_stmt_fetch(stmt);
+	proxy->data_seek(row_position - 1);
+	proxy->fetch();
 }
 /* }}} */
 
