@@ -1,5 +1,5 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2007 - 2008 MySQL AB, 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -11,7 +11,9 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <sstream>
+#include <boost/scoped_array.hpp>
 
 #include <cppconn/exception.h>
 #include <cppconn/resultset.h>
@@ -20,7 +22,8 @@
 #include "mysql_resultset.h"
 #include "mysql_resultset_metadata.h"
 #include "mysql_statement.h"
-#include "mysql_res_wrapper.h"
+
+#include "nativeapi/native_resultset_wrapper.h"
 
 #include "mysql_debug.h"
 
@@ -32,22 +35,27 @@ namespace mysql
 
 
 /* {{{ MySQL_ResultSet::MySQL_ResultSet() -I- */
-MySQL_ResultSet::MySQL_ResultSet(
-			MYSQL_RES_Wrapper * res, sql::ResultSet::enum_type rset_type,
-			MySQL_Statement * par, sql::mysql::util::my_shared_ptr< MySQL_DebugLogger > * l
+MySQL_ResultSet::MySQL_ResultSet(boost::shared_ptr< NativeAPI::NativeResultsetWrapper > res, sql::ResultSet::enum_type rset_type,
+			MySQL_Statement * par, boost::shared_ptr< MySQL_DebugLogger > & l
 		)
 	: row(NULL), result(res), row_position(0), was_null(false), parent(par),
-	  logger(l? l->getReference():NULL), resultset_type(rset_type)
+	  logger(l), resultset_type(rset_type)
 {
 	CPP_ENTER("MySQL_ResultSet::MySQL_ResultSet");
-	num_rows = mysql_num_rows(result->get());
+	num_rows = result->num_rows();
 
-	num_fields = mysql_num_fields(result->get());
+	num_fields = result->num_fields();
 	for (unsigned int i = 0; i < num_fields; ++i) {
-		sql::mysql::util::my_array_guard< char > upstring(sql::mysql::util::utf8_strup(getFieldMeta(i + 1)->name, 0));
-		field_name_to_index_map[std::string(upstring.get())] = i;
+#if A0
+		std::cout << "Elements=" << field_name_to_index_map.size() << "\n";
+#endif
+		boost::scoped_array< char > upstring(sql::mysql::util::utf8_strup(getFieldMeta(i + 1)->name, 0));
+		field_name_to_index_map[upstring.get()] = i;
 	}
-	rs_meta.reset(new MySQL_ResultSetMetaData(result->getReference(), logger));
+#if A0
+	std::cout << "Elements=" << field_name_to_index_map.size() << "\n";
+#endif
+	rs_meta.reset(new MySQL_ResultSetMetaData(result, logger));
 }
 /* }}} */
 
@@ -55,15 +63,7 @@ MySQL_ResultSet::MySQL_ResultSet(
 /* {{{ MySQL_ResultSet::~MySQL_ResultSet() -I- */
 MySQL_ResultSet::~MySQL_ResultSet()
 {
-	/* Don't remove the block or we can get into problems with logger */
-	{
-		CPP_ENTER("MySQL_ResultSet::~MySQL_ResultSet");
-		if (!isClosed()) {
-			result->dispose();
-		}
-		result->deleteReference();
-	}
-	logger->freeReference();
+	CPP_ENTER("MySQL_ResultSet::~MySQL_ResultSet");
 }
 /* }}} */
 
@@ -95,7 +95,7 @@ MySQL_ResultSet::absolute(const int new_pos)
 		/* According to the JDBC book, absolute(0) means before the result set */
 		row_position = 0;
 		/* no seek() here, as we are not on data*/
-		mysql_data_seek(result->get(), 0);
+		result->data_seek(0);
 	}
 	return (row_position > 0 && row_position < (num_rows + 1));
 }
@@ -120,12 +120,23 @@ MySQL_ResultSet::beforeFirst()
 	CPP_ENTER("MySQL_ResultSet::beforeFirst");
 	checkValid();
 	checkScrollable();
-	mysql_data_seek(result->get(), 0);
+	result->data_seek(0);
 	row_position = 0;
 }
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::cancelRowUpdates() -U- */
+void
+MySQL_ResultSet::cancelRowUpdates()
+{
+	CPP_ENTER("MySQL_ResultSet::cancelRowUpdates");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::cancelRowUpdates()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::checkScrollable() -I- */
 void
 MySQL_ResultSet::checkScrollable() const
@@ -163,26 +174,48 @@ MySQL_ResultSet::checkValid() const
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::clearWarnings() -U- */
+void
+MySQL_ResultSet::clearWarnings()
+{
+	CPP_ENTER("MySQL_ResultSet::clearWarnings");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::clearWarnings()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::close() -I- */
 void
 MySQL_ResultSet::close()
 {
 	CPP_ENTER("MySQL_ResultSet::close");
 	checkValid();
-	result->dispose();
+	result.reset();
+//	result->dispose();
 }
 /* }}} */
 
 
 /* {{{ MySQL_ResultSet::findColumn() -I- */
 uint32_t
-MySQL_ResultSet::findColumn(const std::string& columnLabel) const
+MySQL_ResultSet::findColumn(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::findColumn");
 	checkValid();
-	sql::mysql::util::my_array_guard< char > upstring(sql::mysql::util::utf8_strup(columnLabel.c_str(), 0));
-	FieldNameIndexMap::const_iterator iter = field_name_to_index_map.find(upstring.get());
+	boost::scoped_array< char > upstring(sql::mysql::util::utf8_strup(columnLabel.c_str(), 0));
+#if A0
+	std::cout << "Elements=" << field_name_to_index_map.size() << "\n";
+	FieldNameIndexMap::const_iterator tmp_iter = field_name_to_index_map.begin();
+	FieldNameIndexMap::const_iterator tmp_iter_end = field_name_to_index_map.end();
+	for (;tmp_iter != tmp_iter_end; tmp_iter++) {
+		std::cout << "[[" << tmp_iter->first << "]] second=[[" << tmp_iter->second << "]]\n";
+	}
+	sql::SQLString tmp(upstring.get());
+	std::cout << "[" << tmp << "]\n";
+#endif
 
+	FieldNameIndexMap::const_iterator iter = field_name_to_index_map.find(sql::SQLString(upstring.get()));
 	if (iter == field_name_to_index_map.end()) {
 		return 0;
 	}
@@ -224,7 +257,7 @@ MySQL_ResultSet::getBlob(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ResultSet::getBlob() -I- */
 std::istream *
-MySQL_ResultSet::getBlob(const std::string& columnLabel) const
+MySQL_ResultSet::getBlob(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getBlob(string)");
 	/* isBeforeFirst checks for validity */
@@ -252,7 +285,7 @@ MySQL_ResultSet::getBoolean(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ResultSet::getBoolean() -I- */
 bool
-MySQL_ResultSet::getBoolean(const std::string& columnLabel) const
+MySQL_ResultSet::getBoolean(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getBoolean(string)");
 	/* isBeforeFirst checks for validity */
@@ -264,6 +297,30 @@ MySQL_ResultSet::getBoolean(const std::string& columnLabel) const
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::getConcurrency() -U- */
+int
+MySQL_ResultSet::getConcurrency()
+{
+	CPP_ENTER("MySQL_ResultSet::getConcurrency");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getConcurrency()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::getCursorName() -U- */
+SQLString
+MySQL_ResultSet::getCursorName()
+{
+	CPP_ENTER("MySQL_ResultSet::getCursorName");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getCursorName()");
+	return ""; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::getDouble() -I- */
 long double
 MySQL_ResultSet::getDouble(const uint32_t columnIndex) const
@@ -283,6 +340,9 @@ MySQL_ResultSet::getDouble(const uint32_t columnIndex) const
 		return 0.0;
 	}
 	was_null = false;
+	if (getFieldMeta(columnIndex)->type == MYSQL_TYPE_BIT) {
+		return getInt64(columnIndex);
+	}
 	return sql::mysql::util::strtold(row[columnIndex - 1], NULL);
 }
 /* }}} */
@@ -290,7 +350,7 @@ MySQL_ResultSet::getDouble(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ResultSet::getDouble() -I- */
 long double
-MySQL_ResultSet::getDouble(const std::string& columnLabel) const
+MySQL_ResultSet::getDouble(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getDouble(string)");
 	return getDouble(findColumn(columnLabel));
@@ -298,6 +358,50 @@ MySQL_ResultSet::getDouble(const std::string& columnLabel) const
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::getFetchDirection() -U- */
+int
+MySQL_ResultSet::getFetchDirection()
+{
+	CPP_ENTER("MySQL_ResultSet::getFetchDirection");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getFetchDirection()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::getFetchSize() -U- */
+size_t
+MySQL_ResultSet::getFetchSize()
+{
+	CPP_ENTER("MySQL_ResultSet::getFetchSize");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getFetchSize()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::getFieldMeta() -U- */
+MYSQL_FIELD * MySQL_ResultSet::getFieldMeta(unsigned int columnIndex) const
+{
+	return result->fetch_field_direct(columnIndex - 1);
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::getHoldability() -U- */
+int
+MySQL_ResultSet::getHoldability()
+{
+	CPP_ENTER("MySQL_ResultSet::getHoldability");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getHoldability()");
+	return 0; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::getInt() -I- */
 int32_t
 MySQL_ResultSet::getInt(const uint32_t columnIndex) const
@@ -314,14 +418,14 @@ MySQL_ResultSet::getInt(const uint32_t columnIndex) const
 	if (getFieldMeta(columnIndex)->flags & UNSIGNED_FLAG) {
 		return static_cast<uint32_t>(getInt64(columnIndex));
 	}
-	return static_cast<int32_t>(getInt64(columnIndex));	
+	return static_cast<int32_t>(getInt64(columnIndex));
 }
 /* }}} */
 
 
 /* {{{ MySQL_ResultSet::getInt() -I- */
 int32_t
-MySQL_ResultSet::getInt(const std::string& columnLabel) const
+MySQL_ResultSet::getInt(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getInt(string)");
 	return getInt(findColumn(columnLabel));
@@ -349,7 +453,7 @@ MySQL_ResultSet::getUInt(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ResultSet::getUInt() -I- */
 uint32_t
-MySQL_ResultSet::getUInt(const std::string& columnLabel) const
+MySQL_ResultSet::getUInt(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getUInt(string)");
 	return getUInt(findColumn(columnLabel));
@@ -378,6 +482,26 @@ MySQL_ResultSet::getInt64(const uint32_t columnIndex) const
 	}
 	CPP_INFO_FMT("%ssigned", (getFieldMeta(columnIndex)->flags & UNSIGNED_FLAG)? "un":"");
 	was_null = false;
+	if (getFieldMeta(columnIndex)->type == MYSQL_TYPE_BIT) {
+		uint64_t uval = 0;
+		std::div_t length= std::div(getFieldMeta(columnIndex)->length, 8);
+		if (length.rem) {
+			++length.quot;
+		}
+
+		switch (length.quot) {
+			case 8:uval = (uint64_t) bit_uint8korr(row[columnIndex - 1]);break;
+			case 7:uval = (uint64_t) bit_uint7korr(row[columnIndex - 1]);break;
+			case 6:uval = (uint64_t) bit_uint6korr(row[columnIndex - 1]);break;
+			case 5:uval = (uint64_t) bit_uint5korr(row[columnIndex - 1]);break;
+			case 4:uval = (uint64_t) bit_uint4korr(row[columnIndex - 1]);break;
+			case 3:uval = (uint64_t) bit_uint3korr(row[columnIndex - 1]);break;
+			case 2:uval = (uint64_t) bit_uint2korr(row[columnIndex - 1]);break;
+			case 1:uval = (uint64_t) bit_uint1korr(row[columnIndex - 1]);break;
+		}
+		return uval;
+	}
+
 	if (getFieldMeta(columnIndex)->flags & UNSIGNED_FLAG) {
 		return strtoull(row[columnIndex - 1], NULL, 10);
 	}
@@ -388,7 +512,7 @@ MySQL_ResultSet::getInt64(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ResultSet::getInt64() -I- */
 int64_t
-MySQL_ResultSet::getInt64(const std::string& columnLabel) const
+MySQL_ResultSet::getInt64(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getInt64(string)");
 	return getInt64(findColumn(columnLabel));
@@ -417,17 +541,36 @@ MySQL_ResultSet::getUInt64(const uint32_t columnIndex) const
 	}
 	CPP_INFO_FMT("%ssigned", (getFieldMeta(columnIndex)->flags & UNSIGNED_FLAG)? "un":"");
 	was_null = false;
-	if (getFieldMeta(columnIndex)->flags & UNSIGNED_FLAG) {
-		return strtoull(row[columnIndex - 1], NULL, 10);	
+	if (getFieldMeta(columnIndex)->type == MYSQL_TYPE_BIT) {
+		uint64_t uval = 0;
+		std::div_t length= std::div(getFieldMeta(columnIndex)->length, 8);
+		if (length.rem) {
+			++length.quot;
+		}
+		switch (length.quot) {
+			case 8:uval = (uint64_t) bit_uint8korr(row[columnIndex - 1]);break;
+			case 7:uval = (uint64_t) bit_uint7korr(row[columnIndex - 1]);break;
+			case 6:uval = (uint64_t) bit_uint6korr(row[columnIndex - 1]);break;
+			case 5:uval = (uint64_t) bit_uint5korr(row[columnIndex - 1]);break;
+			case 4:uval = (uint64_t) bit_uint4korr(row[columnIndex - 1]);break;
+			case 3:uval = (uint64_t) bit_uint3korr(row[columnIndex - 1]);break;
+			case 2:uval = (uint64_t) bit_uint2korr(row[columnIndex - 1]);break;
+			case 1:uval = (uint64_t) bit_uint1korr(row[columnIndex - 1]);break;
+		}
+		return uval;
 	}
-	return strtoll(row[columnIndex - 1], NULL, 10);	
+
+	if (getFieldMeta(columnIndex)->flags & UNSIGNED_FLAG) {
+		return strtoull(row[columnIndex - 1], NULL, 10);
+	}
+	return strtoll(row[columnIndex - 1], NULL, 10);
 }
 /* }}} */
 
 
 /* {{{ MySQL_ResultSet::getUInt64() -I- */
 uint64_t
-MySQL_ResultSet::getUInt64(const std::string& columnLabel) const
+MySQL_ResultSet::getUInt64(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getUInt64(string)");
 	return getUInt64(findColumn(columnLabel));
@@ -458,6 +601,30 @@ MySQL_ResultSet::getRow() const
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::getRowId() -U- */
+sql::RowID *
+MySQL_ResultSet::getRowId(uint32_t)
+{
+	CPP_ENTER("MySQL_ResultSet::getRowId");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getRowId(uint32_t columnIndex)");
+	return NULL; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::getRowId() -U- */
+sql::RowID *
+MySQL_ResultSet::getRowId(const sql::SQLString &)
+{
+	CPP_ENTER("MySQL_ResultSet::getRowId");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getRowId(const sql::SQLString & columnLabel)");
+	return NULL; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::getStatement() -I- */
 const sql::Statement *
 MySQL_ResultSet::getStatement() const
@@ -469,7 +636,7 @@ MySQL_ResultSet::getStatement() const
 
 
 /* {{{ MySQL_ResultSet::getString() -I- */
-std::string
+SQLString
 MySQL_ResultSet::getString(const uint32_t columnIndex) const
 {
 	CPP_ENTER("MySQL_ResultSet::getString(int)");
@@ -488,17 +655,24 @@ MySQL_ResultSet::getString(const uint32_t columnIndex) const
 		was_null = true;
 		return "";
 	}
-	size_t len = mysql_fetch_lengths(result->get())[columnIndex - 1];
+
+	if (getFieldMeta(columnIndex)->type == MYSQL_TYPE_BIT) {
+		char buf[30];
+		snprintf(buf, sizeof(buf) - 1, "%llu", (unsigned long long) getUInt64(columnIndex));
+		return sql::SQLString(buf);
+	}
+
+	size_t len = result->fetch_lengths()[columnIndex - 1];
 	CPP_INFO_FMT("value=%*s",  len> 50? 50:len, row[columnIndex - 1]);
 	was_null = false;
-	return std::string(row[columnIndex - 1], len);
+	return sql::SQLString(row[columnIndex - 1], len);
 }
 /* }}} */
 
 
 /* {{{ MySQL_ResultSet::getString() -I- */
-std::string
-MySQL_ResultSet::getString(const std::string& columnLabel) const
+SQLString
+MySQL_ResultSet::getString(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::getString(string)");
 	return getString(findColumn(columnLabel));
@@ -517,6 +691,28 @@ MySQL_ResultSet::getType() const
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::getWarnings() -U- */
+void
+MySQL_ResultSet::getWarnings()
+{
+	CPP_ENTER("MySQL_ResultSet::getWarnings");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::getWarnings()");
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::insertRow() -U- */
+void
+MySQL_ResultSet::insertRow()
+{
+	CPP_ENTER("MySQL_ResultSet::insertRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::insertRow()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::isAfterLast() -I- */
 bool
 MySQL_ResultSet::isAfterLast() const
@@ -545,7 +741,8 @@ bool
 MySQL_ResultSet::isClosed() const
 {
 	CPP_ENTER("MySQL_ResultSet::isClosed");
-	return !result->isValid();
+	return !result;
+//	return !result->isValid();
 }
 /* }}} */
 
@@ -594,7 +791,7 @@ MySQL_ResultSet::isNull(const uint32_t columnIndex) const
 
 /* {{{ MySQL_ResultSet::isNull() -I- */
 bool
-MySQL_ResultSet::isNull(const std::string& columnLabel) const
+MySQL_ResultSet::isNull(const sql::SQLString& columnLabel) const
 {
 	CPP_ENTER("MySQL_ResultSet::isNull(string)");
 	int32_t col_idx = findColumn(columnLabel);
@@ -622,6 +819,30 @@ MySQL_ResultSet::last()
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::moveToCurrentRow() -U- */
+void
+MySQL_ResultSet::moveToCurrentRow()
+{
+	CPP_ENTER("MySQL_ResultSet::moveToCurrentRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::moveToCurrentRow()");
+	checkScrollable();
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::moveToInsertRow() -U- */
+void
+MySQL_ResultSet::moveToInsertRow()
+{
+	CPP_ENTER("MySQL_ResultSet::moveToInsertRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::moveToInsertRow()");
+	checkScrollable();
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::next() -I- */
 bool
 MySQL_ResultSet::next()
@@ -633,14 +854,14 @@ MySQL_ResultSet::next()
 		if (isLast()) {
 			afterLast();
 		} else if (row_position < num_rows + 1) {
-			row = mysql_fetch_row(result->get());
+			row = result->fetch_row();
 			++row_position;
 			ret = (row != NULL);
 		}
 	} else {
-		row = mysql_fetch_row(result->get());
+		row = result->fetch_row();
 		++row_position;
-		ret = (row != NULL);	
+		ret = (row != NULL);
 	}
 	CPP_INFO_FMT("new_position=%llu num_rows=%llu", row_position, num_rows);
 	return ret;
@@ -671,6 +892,17 @@ MySQL_ResultSet::previous()
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::refreshRow() -U- */
+void
+MySQL_ResultSet::refreshRow()
+{
+	CPP_ENTER("MySQL_ResultSet::refreshRow");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::refreshRow()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::relative() -I- */
 bool
 MySQL_ResultSet::relative(const int rows)
@@ -692,6 +924,42 @@ MySQL_ResultSet::relative(const int rows)
 /* }}} */
 
 
+/* {{{ MySQL_ResultSet::rowDeleted() -U- */
+bool
+MySQL_ResultSet::rowDeleted()
+{
+	CPP_ENTER("MySQL_ResultSet::rowDeleted");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::rowDeleted()");
+	return false; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::rowInserted() -U- */
+bool
+MySQL_ResultSet::rowInserted()
+{
+	CPP_ENTER("MySQL_ResultSet::rowInserted");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::rowInserted()");
+	return false; // fool compilers
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::rowUpdated() -U- */
+bool
+MySQL_ResultSet::rowUpdated()
+{
+	CPP_ENTER("MySQL_ResultSet::rowUpdated");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::rowUpdated()");
+	return false; // fool compilers
+}
+/* }}} */
+
+
 /* {{{ MySQL_ResultSet::rowsCount() -I- */
 size_t
 MySQL_ResultSet::rowsCount() const
@@ -699,7 +967,18 @@ MySQL_ResultSet::rowsCount() const
 	CPP_ENTER("MySQL_ResultSet::rowsCount");
 	checkValid();
 	checkScrollable();
-	return (size_t) mysql_num_rows(result->get());
+	return static_cast<size_t>(result->num_rows());
+}
+/* }}} */
+
+
+/* {{{ MySQL_ResultSet::setFetchSize() -U- */
+void
+MySQL_ResultSet::setFetchSize(size_t /* rows */)
+{
+	CPP_ENTER("MySQL_ResultSet::setFetchSize");
+	checkValid();
+	throw sql::MethodNotImplementedException("MySQL_ResultSet::setFetchSize()");
 }
 /* }}} */
 
@@ -736,8 +1015,8 @@ MySQL_ResultSet::seek()
 {
 	CPP_ENTER("MySQL_ResultSet::seek");
 	checkScrollable();
-	mysql_data_seek(result->get(), row_position - 1);
-	row = mysql_fetch_row(result->get());
+	result->data_seek(row_position - 1);
+	row = result->fetch_row();
 }
 /* }}} */
 
