@@ -1,5 +1,5 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2007 - 2008 MySQL AB, 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -19,7 +19,7 @@ namespace sql
 {
 namespace mysql
 {
-	const std::string & errCode2SqlState(int errCode);
+	const sql::SQLString & errCode2SqlState(int errCode);
 
 	sql::SQLWarning * loadMysqlWarnings(sql::Connection * connection);
 
