@@ -753,7 +753,7 @@ sql::Connection * BaseTestFixture::getConnection()
   //props.setProperty("runningCTS13", "true");
   if (driver == NULL)
   {
-    driver=get_driver_instance();
+    driver= sql::mysql::get_driver_instance();
     logMsg(String(_T("Done: loaded ")) + driver->getName());
   }
 
@@ -804,27 +804,6 @@ void BaseTestFixture::tearDown()
 
 }
 
-/**
- * Checks whether the database we're connected to meets the given version
- * minimum
- *
- * @param major
- *            the major version to meet
- * @param minor
- *            the minor version to meet
- *
- * @return bool if the major/minor is met
- *
- * @throws SQLException &
- *             if an error occurs.
- */
-
-/* throws SQLException & */
-
-bool BaseTestFixture::versionMeetsMinimum(int major, int minor)
-{
-  return versionMeetsMinimum(major, minor, 0);
-}
 
 /**
  * Checks whether the database we're connected to meets the given version
@@ -843,9 +822,12 @@ bool BaseTestFixture::versionMeetsMinimum(int major, int minor)
 
 /* throws SQLException & */
 
-bool BaseTestFixture::versionMeetsMinimum(int major, int minor, int subminor)
+bool BaseTestFixture::versionMeetsMinimum(unsigned int major, unsigned int minor, unsigned int subminor)
 {
-  return true;
+
+  return ( conn->getMetaData()->getDatabaseMajorVersion() >= major
+        && conn->getMetaData()->getDatabaseMinorVersion() >= minor
+        && conn->getMetaData()->getDatabasePatchVersion() >= subminor );
 
   /*(dynamic_cast<sql::mysql::MySQL_Connection*> (this->conn))->versionMeetsMinimum(
   major, minor, subminor);*/
