@@ -1,5 +1,5 @@
 /*
-   Copyright 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -12,13 +12,13 @@
 #ifndef __CCPP_SINGLETON_H
 #define __CCPP_SINGLETON_H
 
-#include "nocopy.h"
+#include <boost/noncopyable.hpp>
 
 namespace policies
 {
 
 template <class T>
-class Singleton: public nocopy
+class Singleton: public boost::noncopyable
 {
 protected:
 
