@@ -1,5 +1,5 @@
 /*
-   Copyright 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -18,7 +18,19 @@
 #include "test_listener.h"
 #include "test_runner.h"
 #include "test_factory.h"
+#include "test_timer.h"
 
+#ifdef _WIN32
+
+#include <windows.h>
+#define SLEEP(x) Sleep((x)*1000)
+
+#else
+
+#include <unistd.h>
+#define SLEEP(x) sleep((x))
+
+#endif
 
 #define REGISTER_FIXTURE( theFixtureClass ) \
   \
