@@ -1,5 +1,5 @@
 /*
-   Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2009 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -13,7 +13,7 @@
 #define __TESTCONTAINER_H_
 
 #include "../common/ccppTypes.h"
-#include "../common/nocopy.h"
+#include <boost/noncopyable.hpp>
 
 
 namespace testsuite
@@ -26,7 +26,8 @@ namespace testsuite
     class TestContainer
     {
     protected:
-      class StorableTest : public policies::nocopy
+
+    class StorableTest : public boost::noncopyable
       {
         StorableTest(){}
 
@@ -38,11 +39,7 @@ namespace testsuite
 
         StorableTest( Test & test2decorate );
 
-        void            runTest ();
-
-        const String &  name    () const;
-
-        /*Test & operator * ();
+         /*Test & operator * ();
         Test * operator ->();*/
 
         Test * get();
