@@ -1,5 +1,5 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2007 - 2008 MySQL AB, 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -19,29 +19,14 @@ int loops = 2;
 static sql::Connection *
 get_connection(const std::string& host, const std::string& user, const std::string& pass)
 {
-	static sql::Driver * driver = get_driver_instance();
+	static sql::Driver * driver = sql::mysql::get_driver_instance();
 	if (loops % 2) {
 		return driver->connect(host, /*port,*/ user, pass);
 	} else {
-		std::map<std::string, sql::ConnectPropertyVal> connection_properties;
-		{
-			sql::ConnectPropertyVal tmp;
-			tmp.str.val = host.c_str();
-			tmp.str.len = host.length();
-			connection_properties[std::string("hostName")] = tmp;
-		}
-		{
-			sql::ConnectPropertyVal tmp;
-			tmp.str.val = user.c_str();
-			tmp.str.len = user.length();
-			connection_properties[std::string("userName")] = tmp;
-		}
-		{
-			sql::ConnectPropertyVal tmp;
-			tmp.str.val = pass.c_str();
-			tmp.str.len = pass.length();
-			connection_properties[std::string("password")] = tmp;
-		}
+		sql::ConnectOptionsMap connection_properties;
+		connection_properties["hostName"] = host;
+		connection_properties["userName"] = user;
+		connection_properties["password"] = pass;
 		return new sql::mysql::MySQL_Connection(connection_properties);
 	}
 }
