@@ -1,6 +1,6 @@
 
 /*
-   Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2009 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    The MySQL Connector/C++ is licensed under the terms of the GPL
    <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -16,6 +16,9 @@
 #include "preparedstatement.h"
 #include <stdlib.h>
 
+#include <boost/scoped_ptr.hpp>
+#include <boost/scoped_array.hpp>
+
 namespace testsuite
 {
 namespace classes
@@ -29,6 +32,7 @@ void preparedstatement::InsertSelectAllTypes()
   std::vector<columndefinition>::iterator it;
   stmt.reset(con->createStatement());
   bool got_warning=false;
+  size_t len;
 
   try
   {
@@ -62,6 +66,11 @@ void preparedstatement::InsertSelectAllTypes()
       checkResultSetScrolling(res);
       ASSERT(res->next());
 
+      res.reset();
+      res.reset(pstmt->executeQuery());
+      checkResultSetScrolling(res);
+      ASSERT(res->next());
+
       if (it->check_as_string && (res->getString(1) != it->as_string))
       {
         sql.str("");
@@ -342,6 +351,81 @@ void preparedstatement::InsertSelectAllTypes()
       {
       }
       res->first();
+
+      // TODO - make BLOB
+
+      if (it->check_as_string)
+      {
+        {
+          boost::scoped_ptr<std::istream> blob_output_stream(res->getBlob(1));
+          len=it->as_string.length();
+          boost::scoped_array<char> blob_out(new char[len]);
+          blob_output_stream->read(blob_out.get(), len);
+          if (it->as_string.compare(0, blob_output_stream->gcount()
+                                    , blob_out.get(), blob_output_stream->gcount()))
+          {
+            sql.str("");
+            sql << "... \t\tWARNING - SQL: '" << it->sqldef << "' - expecting '" << it->as_string << "'";
+            sql << " got '" << res->getString(1) << "'";
+            logMsg(sql.str());
+            got_warning=true;
+          }
+        }
+
+        {
+          boost::scoped_ptr<std::istream> blob_output_stream(res->getBlob("id"));
+          len=it->as_string.length();
+          boost::scoped_array<char> blob_out(new char[len]);
+          blob_output_stream->read(blob_out.get(), len);
+          if (it->as_string.compare(0, blob_output_stream->gcount()
+                                    , blob_out.get(), blob_output_stream->gcount()))
+          {
+            sql.str("");
+            sql << "... \t\tWARNING - SQL: '" << it->sqldef << "' - expecting '" << it->as_string << "'";
+            sql << " got '" << res->getString(1) << "'";
+            logMsg(sql.str());
+            got_warning=true;
+          }
+        }
+      }
+      try
+      {
+        res->getBlob(0);
+        FAIL("Invalid argument not detected");
+      }
+      catch (sql::InvalidArgumentException &)
+      {
+      }
+
+      try
+      {
+        res->getBlob(3);
+        FAIL("Invalid argument not detected");
+      }
+      catch (sql::InvalidArgumentException &)
+      {
+      }
+
+      res->beforeFirst();
+      try
+      {
+        res->getBlob(1);
+        FAIL("Invalid argument not detected");
+      }
+      catch (sql::InvalidArgumentException &)
+      {
+      }
+      res->afterLast();
+      try
+      {
+        res->getBlob(1);
+        FAIL("Invalid argument not detected");
+      }
+      catch (sql::InvalidArgumentException &)
+      {
+      }
+      res->first();
+
     }
     stmt->execute("DROP TABLE IF EXISTS test");
     if (got_warning)
@@ -635,16 +719,42 @@ void preparedstatement::assortedSetType()
       {
       }
 
+      pstmt->clearParameters();
+      std::stringstream blob_input_stream;
+      blob_input_stream.str(it->value);
+      pstmt->setBlob(1, &blob_input_stream);
+      ASSERT_EQUALS(1, pstmt->executeUpdate());
+
+      pstmt->clearParameters();
+      try
+      {
+        pstmt->setBlob(0, &blob_input_stream);
+        FAIL("Invalid argument not detected");
+      }
+      catch (sql::InvalidArgumentException)
+      {
+      }
+
+      pstmt->clearParameters();
+      try
+      {
+        pstmt->setBlob(2, &blob_input_stream);
+        FAIL("Invalid argument not detected");
+      }
+      catch (sql::InvalidArgumentException)
+      {
+      }
+
       pstmt.reset(con->prepareStatement("SELECT COUNT(IFNULL(id, 1)) AS _num FROM test"));
       res.reset(pstmt->executeQuery());
       checkResultSetScrolling(res);
       ASSERT(res->next());
-      if (res->getInt("_num") != (10 + (int) it->is_nullable))
+      if (res->getInt("_num") != (11 + (int) it->is_nullable))
       {
         sql.str("");
         sql << "....\t\tWARNING, SQL: " << it->sqldef << ", nullable " << std::boolalpha;
         sql << it->is_nullable << ", found " << res->getInt(1) << "columns but";
-        sql << " expecting " << (10 + (int) it->is_nullable);
+        sql << " expecting " << (11 + (int) it->is_nullable);
         logMsg(sql.str());
         got_warning=true;
       }
@@ -819,27 +929,82 @@ void preparedstatement::getMetaData()
   }
 }
 
-void preparedstatement::callSP()
+bool preparedstatement::createSP(std::string sp_code)
 {
-  logMsg("preparedstatement::callSP() - MySQL_PreparedStatement::*()");
+  try
+  {
+    stmt.reset(con->createStatement());
+    stmt->execute("DROP PROCEDURE IF EXISTS p");
+  }
+  catch (sql::SQLException &e)
+  {
+    logMsg(e.what());
+    return false;
+  }
 
   try
   {
-    try
+    pstmt.reset(con->prepareStatement(sp_code));
+    ASSERT(!pstmt->execute());
+    logMsg("... using PS for everything");
+  }
+  catch (sql::SQLException &e)
+  {
+    if (e.getErrorCode() != 1295)
     {
-      pstmt.reset(con->prepareStatement("DROP PROCEDURE IF EXISTS p"));
-      pstmt->execute();
+      logErr(e.what());
+      std::stringstream msg;
+      msg.str("");
+      msg << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+      logErr(msg.str());
+      fail(e.what(), __FILE__, __LINE__);
     }
-    catch (sql::SQLException &e)
+    stmt->execute(sp_code);
+  }
+
+  return true;
+}
+
+void preparedstatement::callSP()
+{
+  logMsg("preparedstatement::callSP() - MySQL_PreparedStatement::*()");
+  std::string sp_code("CREATE PROCEDURE p(OUT ver_param VARCHAR(25)) BEGIN SELECT VERSION() INTO ver_param; END;");
+  try
+  {
+    if (!createSP(sp_code))
     {
       logMsg("... skipping:");
-      logMsg(e.what());
       return;
     }
 
     DatabaseMetaData * dbmeta=con->getMetaData();
+    try
+    {
+      pstmt.reset(con->prepareStatement("CALL p(@version)"));
+      ASSERT(!pstmt->execute());
+      ASSERT(!pstmt->execute());
+    }
+    catch (sql::SQLException &e)
+    {
+      if (e.getErrorCode() != 1295)
+      {
+        logErr(e.what());
+        std::stringstream msg;
+        msg.str("");
+        msg << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+        logErr(msg.str());
+        fail(e.what(), __FILE__, __LINE__);
+      }
+      // PS protocol does not support CALL
+      return;
+    }
 
-    pstmt.reset(con->prepareStatement("CREATE PROCEDURE p(OUT ver_param VARCHAR(25)) BEGIN SELECT VERSION() INTO ver_param; END;"));
+    pstmt.reset(con->prepareStatement("SELECT @version AS _version"));
+    res.reset(pstmt->executeQuery());
+    ASSERT(res->next());
+    ASSERT_EQUALS(dbmeta->getDatabaseProductVersion(), res->getString("_version"));
+
+    pstmt.reset(con->prepareStatement("SET @version='no_version'"));
     ASSERT(!pstmt->execute());
     pstmt.reset(con->prepareStatement("CALL p(@version)"));
     ASSERT(!pstmt->execute());
@@ -851,10 +1016,203 @@ void preparedstatement::callSP()
   catch (sql::SQLException &e)
   {
     logErr(e.what());
-    logErr("SQLState: " + std::string(e.getSQLState()));
+    std::stringstream msg;
+    msg.str("");
+    msg << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+    logErr(msg.str());
     fail(e.what(), __FILE__, __LINE__);
   }
+}
 
+void preparedstatement::callSPInOut()
+{
+  logMsg("preparedstatement::callSPInOut() - MySQL_PreparedStatement::*()");
+  std::string sp_code("CREATE PROCEDURE p(IN ver_in VARCHAR(25), OUT ver_out VARCHAR(25)) BEGIN SELECT ver_in INTO ver_out; END;");
+  try
+  {
+    if (!createSP(sp_code))
+    {
+      logMsg("... skipping: cannot create SP");
+      return;
+    }
+    try
+    {
+      pstmt.reset(con->prepareStatement("CALL p('myver', @version)"));
+      ASSERT(!pstmt->execute());
+    }
+    catch (sql::SQLException &e)
+    {
+      if (e.getErrorCode() != 1295)
+      {
+        logErr(e.what());
+        std::stringstream msg1;
+        msg1.str("");
+        msg1 << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+        logErr(msg1.str());
+        fail(e.what(), __FILE__, __LINE__);
+      }
+      // PS protocol does not support CALL
+      logMsg("... skipping: PS protocol does not support CALL");
+      return;
+    }
+    pstmt.reset(con->prepareStatement("SELECT @version AS _version"));
+    res.reset(pstmt->executeQuery());
+    ASSERT(res->next());
+    ASSERT_EQUALS("myver", res->getString("_version"));
+  }
+  catch (sql::SQLException &e)
+  {
+    logErr(e.what());
+    std::stringstream msg2;
+    msg2.str("");
+    msg2 << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+    logErr(msg2.str());
+    fail(e.what(), __FILE__, __LINE__);
+  }
+}
+
+void preparedstatement::callSPWithPS()
+{
+  logMsg("preparedstatement::callSPWithPS() - MySQL_PreparedStatement::*()");
+
+  try
+  {
+
+    int mysql_version=getMySQLVersion(con);
+    if (mysql_version < 60000)
+      SKIP("http://bugs.mysql.com/bug.php?id=44495 - Server crash");
+
+    std::string sp_code("CREATE PROCEDURE p(IN val VARCHAR(25)) BEGIN SET @sql = CONCAT('SELECT \"', val, '\"'); PREPARE stmt FROM @sql; EXECUTE stmt; DROP PREPARE stmt; END;");
+    if (!createSP(sp_code))
+    {
+      logMsg("... skipping:");
+      return;
+    }
+
+    try
+    {
+      pstmt.reset(con->prepareStatement("CALL p('abc')"));
+      res.reset(pstmt->executeQuery());
+    }
+    catch (sql::SQLException &e)
+    {
+      if (e.getErrorCode() != 1295)
+      {
+        logErr(e.what());
+        std::stringstream msg1;
+        msg1.str("");
+        msg1 << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+        logErr(msg1.str());
+        fail(e.what(), __FILE__, __LINE__);
+      }
+      // PS interface cannot call this kind of statement
+      return;
+    }
+    ASSERT(res->next());
+    ASSERT_EQUALS("abc", res->getString(1));
+    std::stringstream msg2;
+    msg2.str("");
+    msg2 << "... val = '" << res->getString(1) << "'";
+    logMsg(msg2.str());
+
+    try
+    {
+      pstmt.reset(con->prepareStatement("CALL p(?)"));
+      pstmt->setString(1, "123");
+      res.reset(pstmt->executeQuery());
+    }
+    catch (sql::SQLException &e)
+    {
+      if (e.getErrorCode() != 1295)
+      {
+        logErr(e.what());
+        std::stringstream msg3;
+        msg3.str("");
+        msg3 << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+        logErr(msg3.str());
+        fail(e.what(), __FILE__, __LINE__);
+      }
+      // PS interface cannot call this kind of statement
+      return;
+    }
+    res->close();
+  }
+  catch (sql::SQLException &e)
+  {
+    if (e.getErrorCode() != 1295)
+    {
+      logErr(e.what());
+      std::stringstream msg4;
+      msg4.str("");
+      msg4 << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+      logErr(msg4.str());
+      fail(e.what(), __FILE__, __LINE__);
+    }
+  }
+}
+
+void preparedstatement::callSPMultiRes()
+{
+  logMsg("preparedstatement::callSPMultiRes() - MySQL_PreparedStatement::*()");
+
+  int mysql_version=getMySQLVersion(con);
+  if (mysql_version < 60008)
+    SKIP("http://bugs.mysql.com/bug.php?id=44521 - Server crash");
+
+
+  try
+  {
+    std::string sp_code("CREATE PROCEDURE p() BEGIN SELECT 1; SELECT 2; SELECT 3; END;");
+    if (!createSP(sp_code))
+    {
+      logMsg("... skipping:");
+      return;
+    }
+
+    try
+    {
+      pstmt.reset(con->prepareStatement("CALL p()"));
+      ASSERT(pstmt->execute());
+    }
+    catch (sql::SQLException &e)
+    {
+      if (e.getErrorCode() != 1295)
+      {
+        logErr(e.what());
+        std::stringstream msg1;
+        msg1.str("");
+        msg1 << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+        logErr(msg1.str());
+        fail(e.what(), __FILE__, __LINE__);
+      }
+      // PS interface cannot call this kind of statement
+      return;
+    }
+
+    // Should not work prior to MySQL 6.0
+    std::stringstream msg2;
+    msg2.str("");
+    do
+    {
+      res.reset(pstmt->getResultSet());
+    while (res->next())
+    {
+      msg2 << res->getString(1);
+    }
+    }
+    while (pstmt->getMoreResults());
+
+    ASSERT_EQUALS("123", msg2.str());
+  }
+  catch (sql::SQLException &e)
+  {
+    logErr(e.what());
+    std::stringstream msg3;
+    msg3.str("");
+    msg3 << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+    logErr(msg3.str());
+    fail(e.what(), __FILE__, __LINE__);
+  }
 }
 
 void preparedstatement::anonymousSelect()
@@ -883,14 +1241,13 @@ void preparedstatement::anonymousSelect()
 
 void preparedstatement::crash()
 {
-  bool trace_on=true;
-  // Can hit server bug http://bugs.mysql.com/bug.php?id=43833
-
   logMsg("preparedstatement::crash() - MySQL_PreparedStatement::*");
+
   try
   {
-    trace_on=true;
-    con->setClientOption("clientTrace", &trace_on);
+    int mysql_version=getMySQLVersion(con);
+    if ((mysql_version > 50000 && mysql_version < 50082) || (mysql_version > 51000 && mysql_version < 51035) || (mysql_version > 60000 && mysql_version < 60012))
+      SKIP("http://bugs.mysql.com/bug.php?id=43833 - Server crash");
 
     stmt.reset(con->createStatement());
     stmt->execute("DROP TABLE IF EXISTS test");
@@ -900,9 +1257,6 @@ void preparedstatement::crash()
     pstmt->clearParameters();
     pstmt->setDouble(1, (double) 1.23);
     ASSERT_EQUALS(1, pstmt->executeUpdate());
-
-    trace_on=false;
-    con->setClientOption("clientTrace", &trace_on);
   }
   catch (sql::SQLException &e)
   {
@@ -985,5 +1339,163 @@ void preparedstatement::getWarnings()
   }
 }
 
+void preparedstatement::blob()
+{
+  logMsg("preparedstatement::blob() - MySQL_PreparedStatement::*");
+  char blob_input[512];
+  std::stringstream blob_input_stream;
+  std::stringstream msg;
+  char blob_output[512];
+  int id;
+  int offset=0;
+
+  try
+  {
+    pstmt.reset(con->prepareStatement("DROP TABLE IF EXISTS test"));
+    pstmt->execute();
+
+    pstmt.reset(con->prepareStatement("CREATE TABLE test(id INT, col1 TINYBLOB, col2 TINYBLOB)"));
+    pstmt->execute();
+
+    // Most basic INSERT/SELECT...
+    pstmt.reset(con->prepareStatement("INSERT INTO test(id, col1) VALUES (?, ?)"));
+
+    for (char ascii_code=CHAR_MIN; ascii_code < CHAR_MAX; ascii_code++)
+    {
+      blob_output[offset]='\0';
+      blob_input[offset++]=ascii_code;
+    }
+    blob_input[offset]='\0';
+    blob_output[offset]='\0';
+    for (char ascii_code=CHAR_MAX; ascii_code > CHAR_MIN; ascii_code--)
+    {
+      blob_output[offset]='\0';
+      blob_input[offset++]=ascii_code;
+    }
+    blob_input[offset]='\0';
+    blob_output[offset]='\0';
+
+    id=1;
+    blob_input_stream << blob_input;
+
+    pstmt->setInt(1, id);
+    pstmt->setBlob(2, &blob_input_stream);
+    try
+    {
+      pstmt->setBlob(3, &blob_input_stream);
+      FAIL("Invalid index not detected");
+    }
+    catch (sql::SQLException)
+    {
+    }
+
+    pstmt->execute();
+    pstmt.reset(con->prepareStatement("SELECT id, col1 FROM test WHERE id = 1"));
+    res.reset(pstmt->executeQuery());
+
+    ASSERT(res->next());
+
+    msg.str("");
+    msg << "... simple INSERT/SELECT, '" << blob_input << "' =? '" << res->getString(2) << "'";
+    logMsg(msg.str());
+
+    ASSERT_EQUALS(res->getInt(1), id);
+    ASSERT_EQUALS(res->getString(2), blob_input_stream.str());
+    ASSERT_EQUALS(res->getString(2), blob_input);
+    ASSERT_EQUALS(res->getString("col1"), blob_input_stream.str());
+    ASSERT_EQUALS(res->getString("col1"), blob_input);
+
+    std::auto_ptr< std::istream > blob_output_stream(res->getBlob(2));
+    blob_output_stream->seekg(std::ios::beg);
+    blob_output_stream->get(blob_output, offset + 1);
+    ASSERT_EQUALS(blob_input_stream.str(), blob_output);
+
+    blob_output_stream.reset(res->getBlob("col1"));
+    blob_output_stream->seekg(std::ios::beg);
+    blob_output_stream->get(blob_output, offset + 1);
+    ASSERT_EQUALS(blob_input, blob_output);
+
+    msg.str("");
+    msg << "... second check, '" << blob_input << "' =? '" << blob_output << "'";
+    logMsg(msg.str());
+
+    ASSERT(!res->next());
+    res->close();
+
+    msg.str("");
+    // Data is too long to be stored in a TINYBLOB column
+    msg << "... this is more than TINYBLOB can hold: ";
+    msg << "01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789";
+    pstmt.reset(con->prepareStatement("INSERT INTO test(id, col1) VALUES (?, ?)"));
+    id=2;
+    pstmt->setInt(1, id);
+    pstmt->setBlob(2, &msg);
+    pstmt->execute();
+    pstmt.reset(con->prepareStatement("SELECT id, col1 FROM test WHERE id = 2"));
+    res.reset(pstmt->executeQuery());
+    ASSERT(res->next());
+    ASSERT_EQUALS(res->getInt(1), id);
+    ASSERT_GT((int) (res->getString(2).length()), (int) (msg.str().length()));
+    ASSERT(!res->next());
+    res->close();
+
+    msg << "- what has happened to the stream?";
+    logMsg(msg.str());
+
+
+    pstmt.reset(con->prepareStatement("DROP TABLE IF EXISTS test"));
+    pstmt->execute();
+
+  }
+  catch (sql::SQLException &e)
+  {
+    logErr(e.what());
+    logErr("SQLState: " + std::string(e.getSQLState()));
+    fail(e.what(), __FILE__, __LINE__);
+  }
+
+}
+
+void preparedstatement::executeQuery()
+{
+  logMsg("preparedstatement::executeQuery() - MySQL_PreparedStatement::executeQuery");
+  try
+  {
+    const sql::SQLString option("defaultPreparedStatementResultType");
+    int value=sql::ResultSet::TYPE_FORWARD_ONLY;
+    con->setClientOption(option, static_cast<void *> (&value));
+  }
+  catch (sql::MethodNotImplementedException &e)
+  {
+    /* not available */
+    return;
+  }
+  try
+  {
+
+    stmt.reset(con->createStatement());
+    stmt->execute("DROP TABLE IF EXISTS test");
+    stmt->execute("CREATE TABLE test(id INT UNSIGNED)");
+    stmt->execute("INSERT INTO test(id) VALUES (1), (2), (3)");
+
+    pstmt.reset(con->prepareStatement("SELECT id FROM test ORDER BY id ASC"));
+    res.reset(pstmt->executeQuery());
+    ASSERT(res->next());
+    ASSERT_EQUALS(res->getInt("id"), 1);
+
+    pstmt.reset(con->prepareStatement("DROP TABLE IF EXISTS test"));
+    pstmt->execute();
+  }
+  catch (sql::SQLException &e)
+  {
+    logErr(e.what());
+    std::stringstream msg;
+    msg.str("");
+    msg << "SQLState: " << e.getSQLState() << ", MySQL error code: " << e.getErrorCode();
+    logErr(msg.str());
+    fail(e.what(), __FILE__, __LINE__);
+  }
+}
+
 } /* namespace preparedstatement */
 } /* namespace testsuite */
