@@ -863,41 +863,24 @@ void resultset::fetchBigint()
 void resultset::getResultSetType()
 {
 
-  std::map<std::string, sql::ConnectPropertyVal> connection_properties;
+  sql::ConnectOptionsMap connection_properties;
 
   logMsg("resultset::getResultSetType - MySQL_ResultSet::*");
   try
   {
 
-    {
-      sql::ConnectPropertyVal tmp;
-      /* url comes from the unit testing framework */
-      tmp.str.val=url.c_str();
-      tmp.str.len=url.length();
-      connection_properties[std::string("hostName")]=tmp;
-    }
+    /* url comes from the unit testing framework */
+    connection_properties["hostName"]=url;
 
-    {
-      sql::ConnectPropertyVal tmp;
-      /* user comes from the unit testing framework */
-      tmp.str.val=user.c_str();
-      tmp.str.len=user.length();
-      connection_properties[std::string("userName")]=tmp;
-    }
+    /* user comes from the unit testing framework */
+    connection_properties["userName"]=user;
 
-    {
-      sql::ConnectPropertyVal tmp;
-      tmp.str.val=passwd.c_str();
-      tmp.str.len=passwd.length();
-      connection_properties[std::string("password")]=tmp;
-    }
+    connection_properties["password"]=passwd;
 
     connection_properties.erase("defaultStatementResultType");
     {
       logMsg("... testing defaultStatementResultType");
-      sql::ConnectPropertyVal tmp;
-      tmp.lval=sql::ResultSet::TYPE_FORWARD_ONLY;
-      connection_properties[std::string("defaultStatementResultType")]=tmp;
+      connection_properties["defaultStatementResultType"]= sql::ResultSet::TYPE_FORWARD_ONLY;
       try
       {
         created_objects.clear();
@@ -916,8 +899,7 @@ void resultset::getResultSetType()
       ASSERT_EQUALS(pstmt->getResultSetType(), sql::ResultSet::TYPE_SCROLL_INSENSITIVE);
 
       connection_properties.erase("defaultStatementResultType");
-      tmp.lval=sql::ResultSet::TYPE_SCROLL_INSENSITIVE;
-      connection_properties[std::string("defaultStatementResultType")]=tmp;
+      connection_properties["defaultStatementResultType"]= sql::ResultSet::TYPE_SCROLL_INSENSITIVE;
       try
       {
         created_objects.clear();
@@ -934,8 +916,7 @@ void resultset::getResultSetType()
       ASSERT_EQUALS(pstmt->getResultSetType(), sql::ResultSet::TYPE_SCROLL_INSENSITIVE);
 
       connection_properties.erase("defaultStatementResultType");
-      tmp.lval=sql::ResultSet::TYPE_SCROLL_SENSITIVE;
-      connection_properties[std::string("defaultStatementResultType")]=tmp;
+      connection_properties["defaultStatementResultType"]= sql::ResultSet::TYPE_SCROLL_SENSITIVE;
       try
       {
         created_objects.clear();
