@@ -1,5 +1,5 @@
 /*
-   Copyright 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+   Copyright 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -174,7 +174,10 @@ void unit_fixture::init()
   columns.push_back(columndefinition("TINYBLOB", "TINYTEXT CHARACTER SET binary", sql::DataType::VARBINARY, "a", true, 255, 0, true, "", 255, "NO", false));
   columns.push_back(columndefinition("TINYTEXT", "TINYTEXT CHARACTER SET 'utf8'", sql::DataType::VARCHAR, "a", true, 255, 0, true, "", 255, "NO", false));
   columns.push_back(columndefinition("TINYTEXT", "TINYTEXT CHARACTER SET 'utf8' COLLATE 'utf8_bin'", sql::DataType::VARCHAR, "a", true, 255, 0, true, "", 255, "NO", false));
-
+  columns.push_back(columndefinition("BLOB", "BLOB", sql::DataType::LONGVARBINARY, "a", true, 65535, 0, true, "", 65536, "NO", false));
+  columns.push_back(columndefinition("BLOB", "BLOB NOT NULL", sql::DataType::LONGVARBINARY, "a", true, 65535, 0, false, "", 65535, "NO", false));
+  columns.push_back(columndefinition("TEXT", "TEXT", sql::DataType::LONGVARCHAR, "a", true, 65535, 0, true, "", 65536, "NO", false));
+  columns.push_back(columndefinition("TEXT", "TEXT NOT NULL", sql::DataType::LONGVARCHAR, "a", true, 65535, 0, false, "", 65535, "NO", false));
   columns.push_back(columndefinition("MEDIUMBLOB", "MEDIUMBLOB", sql::DataType::LONGVARBINARY, "a", true, 16777215, 0, true, "", 16777215, "NO", false));
   columns.push_back(columndefinition("MEDIUMBLOB", "MEDIUMBLOB NOT NULL", sql::DataType::LONGVARBINARY, "a", true, 16777215, 0, false, "", 16777215, "NO", false));
   columns.push_back(columndefinition("MEDIUMTEXT", "MEDIUMTEXT", sql::DataType::LONGVARCHAR, "a", true, 16777215, 0, true, "", 16777215, "NO", false));
@@ -285,6 +288,8 @@ void unit_fixture::setUp()
            + " " + String(driver->getMajorVersion() + driver->getMajorVersion + String(".") + driver->getMinorVersion());*/
 
   con->setSchema(db);
+
+  stmt.reset(con->createStatement());
 }
 
 void unit_fixture::tearDown()
@@ -347,37 +352,23 @@ sql::Connection * unit_fixture::getConnection()
 {
   if (driver == NULL)
   {
-    driver=get_driver_instance();
+    driver=sql::mysql::get_driver_instance();
   }
 
-  std::map<std::string, sql::ConnectPropertyVal> connection_properties;
-  {
-    sql::ConnectPropertyVal tmp;
-    /* url comes from the unit testing framework */
-    tmp.str.val=url.c_str();
-    tmp.str.len=url.length();
-    connection_properties[std::string("hostName")]=tmp;
-  }
+  sql::ConnectOptionsMap connection_properties;
+  connection_properties["hostName"]=url;
+  connection_properties["userName"]=user;
+  connection_properties["password"]=passwd;
 
-  {
-    sql::ConnectPropertyVal tmp;
-    /* user comes from the unit testing framework */
-    tmp.str.val=user.c_str();
-    tmp.str.len=user.length();
-    connection_properties[std::string("userName")]=tmp;
-  }
+  bool bval= !TestsRunner::getStartOptions()->getBool("dont-use-is");
+  connection_properties["metadataUseInfoSchema"]=bval;
 
+  bval=TestsRunner::getStartOptions()->getBool("use-dynamic-load");
+  if (bval)
   {
-    sql::ConnectPropertyVal tmp;
-    tmp.str.val=passwd.c_str();
-    tmp.str.len=passwd.length();
-    connection_properties[std::string("password")]=tmp;
-  }
-
-  {
-    sql::ConnectPropertyVal tmp;
-    tmp.bval= !TestsRunner::getStartOptions()->getBool("dont-use-is");
-    connection_properties[std::string("metadataUseInfoSchema")]=tmp;
+    sql::SQLString clientlib(DYNLOAD_MYSQL_LIB);
+    connection_properties["clientlib"]=clientlib;
+    logMsg("Connection using dynamic load of clientlib " DYNLOAD_MYSQL_LIB);
   }
 
   return driver->connect(connection_properties);
@@ -398,6 +389,12 @@ void unit_fixture::logDebug(const String & message)
   logMsg(message);
 }
 
+int unit_fixture::getMySQLVersion(Connection & con)
+{
+  DatabaseMetaData * dbmeta=con->getMetaData();
+  return dbmeta->getDatabaseMajorVersion() * 10000 + dbmeta->getDatabaseMinorVersion() * 1000 + dbmeta->getDatabasePatchVersion();
+}
+
 std::string unit_fixture::exceptionIsOK(sql::SQLException & e)
 {
   return exceptionIsOK(e, "HY000", 0);
